<!--
This is boilerplate HTML which you can use to design a blog post. You can set the title, date and body copy for your post,
and add images, CSS and JavaScript by putting your files in the same folder as your HTML, then compressing them into a Zip.
Once done, you can upload that Zip file to your blog, and the site will extract the contents and create an editable blog post
out of it all.

You don't need to put attachments in a specific directory. The only attachments you can include are those that would normally
be accepted by the admin site (images, audio, video, etc). When zipping up your post, ensure that this file is called post.html.

To get started, look for tags matching the following HTML attributes:

	data-bpfield="title": Put the title of your post within this tag
	data-bpfield="date": Put a human-readable date in this tag
	data-bpfield="body": Put the HTML of your blog post in here. Include any JavaScript via <script> tags

The whole point of this boilerplate however, is to allow you to custom-design your blog posts, so look for the following tag
and put your custom CSS inside it.

	<style data-bpfield="css">...</style>

Please note that content outside the bounds of these tags will be ignored.

Try and keep the HTML of your post's body basic, as it will be turned into Markdown before being published.

Good luck, and happy designing!

-->